pub mod types;

use tool_helper::{Error, Input};
use std::io::BufRead;
use types::{Section};

pub fn scan(input: Input) -> Result<Vec<Section>, Error> {
    process(input.lines().map(|l| l.unwrap()).into_iter());

    Err(Error::not_implemented("readmap::scan"))
}

fn process<F: std::iter::Iterator<Item=String>>(mut line_iter:F) {
    while let Some(line) = line_iter.next() {
        if line.starts_with(".") {
            process_section(line, &mut line_iter);
        }
    }
}

macro_rules! force_next {
    ($split_line:ident, $line_iter:ident, $line:ident) => {
        {
            if let Some(value) = $split_line.next() {
                value
            }
    
            else {
                $line       = $line_iter.next().unwrap();
                $split_line = $line.split_whitespace();
    
                $split_line.next().unwrap()
            }
        }            
    };
}

fn process_section<F: std::iter::Iterator<Item=String>>(mut line: String, line_iter:&mut F) {
    let mut split_line = line.split_whitespace();
    let name = split_line.next().unwrap().to_string();
    let adr  =   u64::from_str_radix(force_next!(split_line, line_iter, line).trim_start_matches("0x"), 16).ok();
    let size = { // If adr is already empty we do not need to check for a size
        if adr.is_some() {
            usize::from_str_radix(force_next!(split_line, line_iter, line).trim_start_matches("0x"), 16).ok()
        }

        else {
            None
        }
    };

    println!("Section: {} @{:?} {:?}", name, adr, size);
}